/*********************************************
Project : ATmega128_MT
Company : Boston University Electronics Design Facility

Chip type           : ATmega128
Program type        : Application
Clock frequency     : 16,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 1024
*********************************************/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "system.h"
#include "my_lcd.h"
#include "isr.h"

#define MAIN

void scan_keys( void);
uint8_t get_key( void);
void init_keys( void);
void motor_init( void);
void set_lcd_background( void);

// global variables used in ISR
uint8_t ui_mode = UI_RUN;	/* initial UI mode, increment to UI_EDIT_A  */


int main( void) {     

	static uint8_t tick_100_hz, tick_10hz;

	static uint8_t ch = BUTTON_MODE;	/* button pressed - initial value to force display update */		   							
	static uint8_t ui_curs = CUR_OFF;	/* initial cursor mode */
	static uint8_t display_needed;	/* flag: display needs update */

	// mode labels (two lines, one per mode)
	// (could be in program memory on AVR)
	const char* mode_label0[] = { "Set A", "     ", " Run " };
	const char* mode_label1[] = { "     ", "Set B", "     " };
	
	//	const char *banner1 = PSTR("test");

	// Port Initialization 
	ports_init();

	// Timer Initialization
	hwtimers_init();
	
	// Enable Interrupts
	sei();
	


#ifdef MAIN
	// LCD Initialization
	lcd_init();

	set_lcd_background();

	// main event loop
	while (1) {  

		if(hwtimer__tick_1khz){    // run at 1kHz
			hwtimer__tick_1khz = 0;

			lcd_update_at_1khz();

			/* time-critical 1kHz code is located in isr.c */

			if( ++tick_100_hz == 10) { /* run at 100 Hz */
				tick_100_hz = 0;
				scan_keys();
			}

			if( ++tick_10hz == 100) { /* run at 10Hz */
				tick_10hz = 0;
				
				// check for button press, process if any
				if( ch || (ch = get_key()) != 0) {
					
					display_needed = 1;	/* update display after key press */
					
					switch( ch) {
					case BUTTON_MODE:	// Middle Key (end key)
						ui_mode = (ui_mode + 1) % UI_NUM_MODES;
						lcd_rowcol( 0, SCOL_MODE);
						lcd_puts( mode_label0[ui_mode]);
						lcd_rowcol( 1, SCOL_MODE);
						lcd_puts( mode_label1[ui_mode]);
						
						// entered set mode?
						if( ui_mode == UI_SET_A || ui_mode == UI_SET_B) {
							RELAY_LOW;

							ui_curs = CUR_MIN;     /* activate cursor; start editing minutes */
							// entered run mode?
						} else {
							ui_curs = CUR_OFF;	/* deactivate cursor if not editing */	
						}
						break;

					case BUTTON_PREV:	// (KEY_LEFT)
						if( ui_curs) {		/* if cursor active */
							ui_curs--;		/* decrement cursor */
							if( !ui_curs)
							ui_curs = 3;	/* wrap to 3 if 0 */
						}
						break;

					case BUTTON_NEXT:	// (KEY_RIGHT)
						if( ui_curs) {		/* if cursor active */
							++ui_curs;		/* increment cursor */
							if( ui_curs > 3)
							ui_curs = 1;	/* wrap to 1 if 3 */
						}
						break;

					case BUTTON_INC:	// (KEY_UP)		

						break;

					case BUTTON_DEC:

						break;
					}
					
					ch = 0;
				} // if( ch || (ch = get_key()) != 0)
				

				
				if( display_needed) {
					display_needed = 0;
					
					// update timer display after every key press
					if( ui_mode == UI_RUN) {	/* run mode, display active timers */
						lcd_rowcol( SROW_TIMER_A, SCOL_TIMER_A);
						lcd_rowcol( SROW_TIMER_B, SCOL_TIMER_B);

						display_needed = 1;
					} else {
						lcd_rowcol( SROW_TIMER_A, SCOL_TIMER_A);
						lcd_rowcol( SROW_TIMER_B, SCOL_TIMER_B);

					}

					// update cursor display
					if( ui_curs) {
						uint8_t r, c;
						// set r, c to start of timer display
						if( ui_mode == UI_SET_A) {
							r = SROW_TIMER_A;
							c = SCOL_TIMER_A;
						} else {
							r = SROW_TIMER_B;
							c = SCOL_TIMER_B;
						}

						// add offset for field to edit
						switch( ui_curs) {
						case 1:
							c += 1;						
							break;
						case 2:
							c += 4;
							break;
						case 3:	
							c += 6;
							break;
						}
						lcd_rowcol( r, c);	/* move cursor */
						lcd_cur_on_blink();		/* turn it on */
					} else
					lcd_cur_off();		/* turn off if not used */

				} /* if( display_needed) */			
			} //if( ++tick_10hz == 100)
		} // if(hwtimer__tick_1khz)
	} // while(1)
#endif	
} // int main( void)


// type-ahead buffer size (must be power of 2)
#define KEY_FIFO_SIZE 8

static uint8_t key_fifo[KEY_FIFO_SIZE];
static uint8_t key_head, key_tail;

void init_keys()
{
	key_head = 0;
	key_tail = 0;
}

// key scanning
void scan_keys()
{
	static uint8_t sw_old, sw_new;
	uint8_t b;

	sw_new = SW_INPUT & SW_BITS;
	if( sw_old != sw_new) {	/* change of state */
		
		for( b=SW_BIT_LO; b<=SW_BIT_HI; b<<=1) {

			if( ((sw_old^sw_new)&b) && (sw_old&b)) {
				key_fifo[key_head] = b;
				key_head = (key_head+1) & (KEY_FIFO_SIZE-1);
			}

		}

		sw_old = sw_new;
	}
}

// return available key from type-ahead buffer or
// zero if none
uint8_t get_key( void)
{
	uint8_t key;

	if( key_head == key_tail)
	return 0;

	key = key_fifo[key_tail];
	key_tail = (key_tail+1) & (KEY_FIFO_SIZE-1);
	return key;
}

void set_lcd_background( void) {
	lcd_rowcol( 0, 0);
	lcd_puts("A>");
	lcd_rowcol( 1, 0);
	lcd_puts("B>");
}

