#include "system.h"
#include "my_lcd.h"


static int8_t my_row=0, my_col=0;  // LCD position varibles


// update the LCD
// call this function once per millisecond
void lcd_update_at_1khz(void)
{
	static uint8_t lcd_timer;
	static uint16_t ch;

// update LCD timer
	if( lcd_timer > 0)
		--lcd_timer;
	else {
		// see if there is an LCD character to write
		if( (ch = lcd_get()) > 0) {
			if( ch & 0x100)
				lcd_command_nobuf_nd( (ch & 0xff));
			else 
				lcd_putc_nobuf_nd( ch); // display one character, move cursor right

		lcd_timer = 5;
		}
	}
}



// FIFO for LCD characters to be displayed
#define LCD_FIFO_SIZE 64

static uint16_t lcd_fifo[LCD_FIFO_SIZE];
static uint8_t lcd_head, lcd_tail;



// stuff a character into the FIFO.
void lcd_putc( uint8_t c)
{
	lcd_fifo[lcd_head] = (uint16_t)c;
	lcd_head = (lcd_head+1) & (LCD_FIFO_SIZE-1);
}



// stuff a LCD command into the FIFO
void lcd_command( uint8_t c)
{
	lcd_fifo[lcd_head] = (uint16_t)c + 0x100;
	lcd_head = (lcd_head+1) & (LCD_FIFO_SIZE-1);
}



// get a character from FIFO or zero if none
uint16_t lcd_get( void)
{
	uint16_t ch;

	if( lcd_head == lcd_tail)
	return 0;

	ch = lcd_fifo[lcd_tail];
	lcd_tail = (lcd_tail+1) & (LCD_FIFO_SIZE-1);
	return ch;
}



// send string to LCD (no auto linefeed)
void lcd_puts(const char *s)
{
	register char c;

	while ( (c = *s++) ) {
		lcd_putc(c);
	}
}



// send string from program memory to LCD (no auto linefeed)
void lcd_puts_p(const char *progmem_s)
{
	register char c;

	while ( (c = pgm_read_byte(progmem_s++)) ) {
		lcd_putc(c);
	}
}


char buffer[12];  // buffer for lcd_putix and lcd_puti



// send a hex value to the LCD
void lcd_putix( const int val)
{
	lcd_puts( itoa( val, buffer, 16));
}



//  send a integer value to the LCD
void lcd_puti( const int val )
{
	lcd_puts( itoa(val, buffer, 10) );
}



// move cursor to specified row and column
void lcd_rowcol(uint8_t row, uint8_t col)
{
	register uint8_t DDRAMAddr;
	// remap lines into proper order
	switch(row)
	{
	case 0:  DDRAMAddr = LCD_START_LINE1+col; break;
	case 1:  DDRAMAddr = LCD_START_LINE2+col; break;
	case 2:  DDRAMAddr = LCD_START_LINE3+col; break;
	case 3:  DDRAMAddr = LCD_START_LINE4+col; break;
	default: DDRAMAddr = LCD_START_LINE1+col;
	}
	// set data address
	lcd_command( (1<<LCD_DDRAM) + DDRAMAddr);

	// set row col positions
	my_row = row;
	my_col = col;
}



// moves cursor by n poisitions left
void lcd_move_cur_left(uint8_t n)	
{
	uint8_t i;

	for (i=0;i<n;i++)
	{
		lcd_command( 0x10);
		--my_col;
	}
}



// moves cursor by n poisitions right
void lcd_move_cur_right(uint8_t n)	
{
	uint8_t i;

	for (i=0;i<n;i++)
	{
		lcd_command( 0x14);
		++my_col;
	}
}



// set cursor to LCD line two first column
void lcd_line2(void)
{
	lcd_command( (1<<LCD_DDRAM)+LCD_START_LINE2);

	my_row = 1;
	my_col = 0;
}



// set cursor to home position 
void lcd_home(void)
{
	lcd_command( 1<<LCD_HOME);

	my_row = 0;
	my_col = 0;
}



// clear LCD and set cursor to home position
void lcd_cls(void)
{
	lcd_command( 1<<LCD_CLR);

	my_row = 0;
	my_col = 0;
}



// get current row position
uint8_t lcd_cur_row( void)
{
	return my_row;
}



// get current column position
uint8_t lcd_cur_col( void)
{
	return my_col;
}



// shifts the entire display n columns to the left
// the cursor follows the display shift
void lcd_shift_left( uint8_t n)
{
	uint8_t i;

	for (i=0;i<n;i++)
	{
		--my_col;
		lcd_command(0x1e);
	}
}



// shifts the entire display n columns to the right
// the cursor follows the display shift
void lcd_shift_right( uint8_t n)	
{
	uint8_t i;

	for (i=0;i<n;i++)
	{
		++my_col;
		lcd_command(0x18);
	}
}




#ifdef CHARSYMBOL
// defines char symbol in CGRAM
/*
const uint8_t backslash[] PROGMEM= 
{
0b00000000,//back slash
0b00010000,
0b00001000,
0b00000100,
0b00000010,
0b00000001,
0b00000000,
0b00000000
};
LCDdefinechar(backslash,0);
*/
void lcd_definechar(const uint8_t *pc,uint8_t char_code){
	uint8_t a, pcc;
	uint16_t i;
	a=(char_code<<3)|0x40;
	for (i=0; i<8; i++){
		pcc=pgm_read_byte(&pc[i]);
		lcd_command(a++);
		lcd_putc(pcc);
		}
}
#endif



// turn on visible cursor
void lcd_cur_on( void)
{
	lcd_command( LCD_DISP_ON_CURSOR_ON);
}



// turn off visible cursor
void lcd_cur_off( void)
{
	lcd_command( LCD_DISP_ON_CURSOR_OFF);
}



// turn on and blink visible cursor 
void lcd_cur_on_blink( void)
{
	lcd_command( LCD_DISP_ON_CURSOR_BLINK);
}



//--------Initialize LCD --------------------------------------------
void lcd_init(void) {
	LCD_DDR |= (LCD_DATA_BITS | LCD_E_BIT | LCD_RS_BIT);


	_delay_ms(16);        /* wait 16ms or more after power-on */

	LCD_PORT = 0x30;
	toggle_e();
	_delay_us(4100);

	LCD_PORT = 0x30;
	toggle_e();
	_delay_us(120);

	LCD_PORT = 0x30;
	toggle_e();
	_delay_ms(5);

	LCD_PORT = 0x20;
	toggle_e();
	_delay_ms(5);

	lcd_command_nobuf( LCD_FUNCTION_4BIT_2LINES);
	lcd_command_nobuf( LCD_DISP_OFF);
	lcd_command_nobuf( 1<<LCD_CLR);
	lcd_command_nobuf( LCD_MODE_DEFAULT);
	lcd_command_nobuf( LCD_DISP_ON_CURSOR_OFF);

	lcd_head = 0;			/* set up FIFO pointers */
	lcd_tail = 0;
}



// toggle the enable bit
void toggle_e() 
{
	LCD_PORT |= LCD_E_BIT;
	_delay_us( 5);
	LCD_PORT &= ~LCD_E_BIT;
}



// send command to LCD without FIFO buffer
void lcd_command_nobuf( uint8_t c)
{
	LCD_PORT = (c & 0xf0) | LCD_DATA_FIXED_BITS;
	toggle_e();
	LCD_PORT = ((c << 4) & 0xf0) | LCD_DATA_FIXED_BITS;
	toggle_e();
	_delay_ms( 5);
}



// send command to LCD without FIFO buffer and no delay
void lcd_command_nobuf_nd( uint8_t c)
{
	LCD_PORT = (c & 0xf0) | LCD_DATA_FIXED_BITS;
	toggle_e();
	LCD_PORT = ((c << 4) & 0xf0) | LCD_DATA_FIXED_BITS;
	toggle_e();
}



// send char to LCD without FIFO buffer and no delay
void lcd_putc_nobuf_nd( uint8_t c)
{
	++my_col;
	LCD_PORT = LCD_RS_BIT | (c & 0xf0) | LCD_DATA_FIXED_BITS;
	toggle_e();
	LCD_PORT = LCD_RS_BIT | ((c << 4) & 0xf0) | LCD_DATA_FIXED_BITS;
	toggle_e();
}
